@@ -1,19 +1,17 @@
 """base revision for 0.5
 
 Revision ID: 19c0846f6344
-Revises: 
+Revises:
 Create Date: 2016-04-11 16:05:34.873288
 
 """
+
 # revision identifiers, used by Alembic.
 revision = '19c0846f6344'
 down_revision = None
 branch_labels = None
 depends_on = None
 
-import sqlalchemy as sa
-from alembic import op
-
 
 def upgrade():
     pass
