@@ -1,7 +1,9 @@
 """Base API handlers"""
+
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
 import json
+import warnings
 from functools import lru_cache
 from http.client import responses
 from urllib.parse import parse_qs, urlencode, urlparse, urlunparse
@@ -12,7 +14,7 @@ from tornado import web
 from .. import orm
 from ..handlers import BaseHandler
 from ..scopes import get_scopes_for
-from ..utils import get_browser_protocol, isoformat, url_escape_path, url_path_join
+from ..utils import isoformat, url_escape_path, url_path_join
 
 PAGINATION_MEDIA_TYPE = "application/jupyterhub-pagination+json"
 
@@ -23,7 +25,6 @@ class APIHandler(BaseHandler):
     Differences from page handlers:
 
     - JSON responses and errors
-    - strict referer checking for Cookie-authenticated requests
     - strict content-security-policy
     - methods for REST API models
     """
@@ -39,7 +40,7 @@ class APIHandler(BaseHandler):
         return 'application/json'
 
     @property
-    @lru_cache()
+    @lru_cache
     def accepts_pagination(self):
         """Return whether the client accepts the pagination preview media type"""
         accept_header = self.request.headers.get("Accept", "")
@@ -49,48 +50,12 @@ class APIHandler(BaseHandler):
         return PAGINATION_MEDIA_TYPE in accepts
 
     def check_referer(self):
-        """Check Origin for cross-site API requests.
-
-        Copied from WebSocket with changes:
-
-        - allow unspecified host/referer (e.g. scripts)
-        """
-        host_header = self.app.forwarded_host_header or "Host"
-        host = self.request.headers.get(host_header)
-        if host and "," in host:
-            host = host.split(",", 1)[0].strip()
-        referer = self.request.headers.get("Referer")
-
-        # If no header is provided, assume it comes from a script/curl.
-        # We are only concerned with cross-site browser stuff here.
-        if not host:
-            self.log.warning("Blocking API request with no host")
-            return False
-        if not referer:
-            self.log.warning("Blocking API request with no referer")
-            return False
-
-        proto = get_browser_protocol(self.request)
-
-        full_host = f"{proto}://{host}{self.hub.base_url}"
-        host_url = urlparse(full_host)
-        referer_url = urlparse(referer)
-        # resolve default ports for http[s]
-        referer_port = referer_url.port or (
-            443 if referer_url.scheme == 'https' else 80
+        """DEPRECATED"""
+        warnings.warn(
+            "check_referer is deprecated in JupyterHub 3.2 and always returns True",
+            DeprecationWarning,
+            stacklevel=2,
         )
-        host_port = host_url.port or (443 if host_url.scheme == 'https' else 80)
-        if (
-            referer_url.scheme != host_url.scheme
-            or referer_url.hostname != host_url.hostname
-            or referer_port != host_port
-            or not (referer_url.path + "/").startswith(host_url.path)
-        ):
-            self.log.warning(
-                f"Blocking Cross Origin API request.  Referer: {referer},"
-                f" {host_header}: {host}, Host URL: {full_host}",
-            )
-            return False
         return True
 
     def check_post_content_type(self):
@@ -111,6 +76,23 @@ class APIHandler(BaseHandler):
 
         return True
 
+    # we also check xsrf on GETs to API endpoints
+    _xsrf_safe_methods = {"HEAD", "OPTIONS"}
+
+    def check_xsrf_cookie(self):
+        if not hasattr(self, '_jupyterhub_user'):
+            # called too early to check if we're token-authenticated
+            return
+        if self._jupyterhub_user is None and 'Origin' not in self.request.headers:
+            # don't raise xsrf if auth failed
+            # don't apply this shortcut to actual cross-site requests, which have an 'Origin' header,
+            # which would reveal if there are credentials present
+            return
+        if getattr(self, '_token_authenticated', False):
+            # if token-authenticated, ignore XSRF
+            return
+        return super().check_xsrf_cookie()
+
     def get_current_user_cookie(self):
         """Extend get_user_cookie to add checks for CORS"""
         cookie_user = super().get_current_user_cookie()
@@ -119,8 +101,6 @@ class APIHandler(BaseHandler):
         # avoiding misleading "Blocking Cross Origin" messages
         # when there's no cookie set anyway.
         if cookie_user:
-            if not self.check_referer():
-                return None
             if (
                 self.request.method.upper() == 'POST'
                 and not self.check_post_content_type()
@@ -213,6 +193,7 @@ class APIHandler(BaseHandler):
 
         model = {
             'name': orm_spawner.name,
+            'full_name': f"{orm_spawner.user.name}/{orm_spawner.name}",
             'last_activity': isoformat(orm_spawner.last_activity),
             'started': isoformat(orm_spawner.started),
             'pending': pending,
@@ -221,7 +202,22 @@ class APIHandler(BaseHandler):
             'url': url_path_join(user.url, url_escape_path(spawner.name), '/'),
             'user_options': spawner.user_options,
             'progress_url': user.progress_url(spawner.name),
+            'full_url': None,
+            'full_progress_url': None,
         }
+        # fill out full_url fields
+        public_url = self.settings.get("public_url")
+        if urlparse(model["url"]).netloc:
+            # if using subdomains, this is already a full URL
+            model["full_url"] = model["url"]
+        if public_url:
+            model["full_progress_url"] = urlunparse(
+                public_url._replace(path=model["progress_url"])
+            )
+            if not model["full_url"]:
+                # set if not defined already by subdomain
+                model["full_url"] = urlunparse(public_url._replace(path=model["url"]))
+
         scope_filter = self.get_scope_filter('admin:server_state')
         if scope_filter(spawner, kind='server'):
             model['state'] = state
@@ -282,17 +278,43 @@ class APIHandler(BaseHandler):
         return self._include_stopped_servers
 
     def user_model(self, user):
-        """Get the JSON model for a User object"""
+        """Get the JSON model for a User object
+
+        User may be either a high-level User wrapper,
+        or a low-level orm.User.
+        """
+        is_orm = False
         if isinstance(user, orm.User):
-            user = self.users[user.id]
+            if user.id in self.users:
+                # if it's an 'active' user, it's in the users dict,
+                # get the wrapper so we can get 'pending' state, etc.
+                user = self.users[user.id]
+            else:
+                # don't create wrapper of low-level orm object
+                is_orm = True
+
+        if is_orm:
+            # if it's not in the users dict,
+            # we know it has no running servers
+            running = False
+            spawners = {}
+        if not is_orm:
+            running = user.running
+            spawners = user.spawners
+
         include_stopped_servers = self.include_stopped_servers
+        # TODO: we shouldn't fetch fields we can't read and then filter them out,
+        # which may be wasted database queries
+        # we should check and then fetch.
+        # but that's tricky for e.g. server filters
+
         model = {
             'kind': 'user',
             'name': user.name,
             'admin': user.admin,
             'roles': [r.name for r in user.roles],
             'groups': [g.name for g in user.groups],
-            'server': user.url if user.running else None,
+            'server': user.url if running else None,
             'pending': None,
             'created': isoformat(user.created),
             'last_activity': isoformat(user.last_activity),
@@ -322,12 +344,12 @@ class APIHandler(BaseHandler):
             model, access_map, user, kind='user', keys=allowed_keys
         )
         if model:
-            if '' in user.spawners and 'pending' in allowed_keys:
-                model['pending'] = user.spawners[''].pending
+            if '' in spawners and 'pending' in allowed_keys:
+                model['pending'] = spawners[''].pending
 
             servers = {}
             scope_filter = self.get_scope_filter('read:servers')
-            for name, spawner in user.spawners.items():
+            for name, spawner in spawners.items():
                 # include 'active' servers, not just ready
                 # (this includes pending events)
                 if (spawner.active or include_stopped_servers) and scope_filter(
@@ -338,6 +360,10 @@ class APIHandler(BaseHandler):
             if include_stopped_servers:
                 # add any stopped servers in the db
                 seen = set(servers.keys())
+                if isinstance(user, orm.User):
+                    # need high-level User wrapper for spawner model
+                    # FIXME: this shouldn't be needed!
+                    user = self.users[user]
                 for name, orm_spawner in user.orm_spawners.items():
                     if name not in seen and scope_filter(orm_spawner, kind='server'):
                         servers[name] = self.server_model(orm_spawner, user=user)
@@ -357,9 +383,10 @@ class APIHandler(BaseHandler):
             'name': group.name,
             'roles': [r.name for r in group.roles],
             'users': [u.name for u in group.users],
+            'properties': group.properties,
         }
         access_map = {
-            'read:groups': {'kind', 'name', 'users'},
+            'read:groups': {'kind', 'name', 'properties', 'users'},
             'read:groups:name': {'kind', 'name'},
             'read:roles:groups': {'kind', 'name', 'roles'},
         }
@@ -408,6 +435,23 @@ class APIHandler(BaseHandler):
 
     _group_model_types = {'name': str, 'users': list, 'roles': list}
 
+    _service_model_types = {
+        'name': str,
+        'admin': bool,
+        'url': str,
+        'oauth_client_allowed_scopes': list,
+        'api_token': str,
+        'info': dict,
+        'display': bool,
+        'oauth_no_confirm': bool,
+        'command': list,
+        'cwd': str,
+        'environment': dict,
+        'user': str,
+        'oauth_client_id': str,
+        'oauth_redirect_uri': str,
+    }
+
     def _check_model(self, model, model_types, name):
         """Check a model provided by a REST API request
 
@@ -417,15 +461,14 @@ class APIHandler(BaseHandler):
             name (str): name of the model, used in error messages
         """
         if not isinstance(model, dict):
-            raise web.HTTPError(400, "Invalid JSON data: %r" % model)
+            raise web.HTTPError(400, f"Invalid JSON data: {model!r}")
         if not set(model).issubset(set(model_types)):
-            raise web.HTTPError(400, "Invalid JSON keys: %r" % model)
+            raise web.HTTPError(400, f"Invalid JSON keys: {model!r}")
         for key, value in model.items():
             if not isinstance(value, model_types[key]):
                 raise web.HTTPError(
                     400,
-                    "%s.%s must be %s, not: %r"
-                    % (name, key, model_types[key], type(value)),
+                    f"{name}.{key} must be {model_types[key]}, not: {type(value)!r}",
                 )
 
     def _check_user_model(self, model):
@@ -446,6 +489,15 @@ class APIHandler(BaseHandler):
                     400, ("group names must be str, not %r", type(groupname))
                 )
 
+    def _check_service_model(self, model):
+        """Check a request-provided service model from a REST API"""
+        self._check_model(model, self._service_model_types, 'service')
+        service_name = model.get('name')
+        if not isinstance(service_name, str):
+            raise web.HTTPError(
+                400, ("Service name must be str, not %r", type(service_name))
+            )
+
     def get_api_pagination(self):
         default_limit = self.settings["api_page_default_limit"]
         max_limit = self.settings["api_page_max_limit"]
@@ -493,7 +545,7 @@ class APIHandler(BaseHandler):
         if next_offset < total_count:
             # if there's a next page
             next_url_parsed = urlparse(self.request.full_url())
-            query = parse_qs(next_url_parsed.query)
+            query = parse_qs(next_url_parsed.query, keep_blank_values=True)
             query['offset'] = [next_offset]
             query['limit'] = [limit]
             next_url_parsed = next_url_parsed._replace(
@@ -517,6 +569,9 @@ class API404(APIHandler):
     Ensures JSON 404 errors for malformed URLs
     """
 
+    def check_xsrf_cookie(self):
+        pass
+
     async def prepare(self):
         await super().prepare()
         raise web.HTTPError(404)
