@@ -1,6 +1,6 @@
 from . import base, login, metrics, pages
-from .base import *
-from .login import *
+from .base import *  # noqa
+from .login import *  # noqa
 
 default_handlers = []
 for mod in (base, pages, login, metrics):
