@@ -1,4 +1,5 @@
 """logging utilities"""
+
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
 import json
@@ -66,7 +67,7 @@ class CoroutineLogFormatter(LogFormatter):
 # url params to be scrubbed if seen
 # any url param that *contains* one of these
 # will be scrubbed from logs
-SCRUB_PARAM_KEYS = ('token', 'auth', 'key', 'code', 'state')
+SCRUB_PARAM_KEYS = ('token', 'auth', 'key', 'code', 'state', '_xsrf')
 
 
 def _scrub_uri(uri):
@@ -105,11 +106,16 @@ def _scrub_headers(headers):
             auth_type = ''
         headers['Authorization'] = f'{auth_type} [secret]'
     if 'Cookie' in headers:
-        c = SimpleCookie(headers['Cookie'])
-        redacted = []
-        for name in c.keys():
-            redacted.append(f"{name}=[secret]")
-        headers['Cookie'] = '; '.join(redacted)
+        try:
+            c = SimpleCookie(headers['Cookie'])
+        except Exception as e:
+            # it's possible for browsers to send invalid cookies
+            headers['Cookie'] = f"Invalid Cookie: {e}"
+        else:
+            redacted = []
+            for name in c.keys():
+                redacted.append(f"{name}=[secret]")
+            headers['Cookie'] = '; '.join(redacted)
     return headers
 
 
