@@ -2,6 +2,7 @@
 
 implements https://oauthlib.readthedocs.io/en/latest/oauth2/server.html
 """
+
 from oauthlib import uri_validate
 from oauthlib.oauth2 import RequestValidator, WebApplicationServer
 from oauthlib.oauth2.rfc6749.grant_types import authorization_code, base
@@ -9,7 +10,12 @@ from tornado.log import app_log
 
 from .. import orm
 from ..roles import roles_to_scopes
-from ..scopes import _check_scopes_exist, access_scopes, identify_scopes
+from ..scopes import (
+    _check_scopes_exist,
+    _resolve_requested_scopes,
+    access_scopes,
+    identify_scopes,
+)
 from ..utils import compare_token, hash_token
 
 # patch absolute-uri check
@@ -150,7 +156,7 @@ class JupyterHubRequestValidator(RequestValidator):
             self.db.query(orm.OAuthClient).filter_by(identifier=client_id).first()
         )
         if orm_client is None:
-            raise ValueError("No such client: %s" % client_id)
+            raise ValueError(f"No such client: {client_id}")
         scopes = set(orm_client.allowed_scopes)
         if 'inherit' not in scopes:
             # add identify-user scope
@@ -249,19 +255,19 @@ class JupyterHubRequestValidator(RequestValidator):
             self.db.query(orm.OAuthClient).filter_by(identifier=client_id).first()
         )
         if orm_client is None:
-            raise ValueError("No such client: %s" % client_id)
+            raise ValueError(f"No such client: {client_id}")
 
         orm_code = orm.OAuthCode(
-            client=orm_client,
             code=code['code'],
             # oauth has 5 minutes to complete
             expires_at=int(orm.OAuthCode.now() + 300),
             scopes=list(request.scopes),
-            user=request.user.orm_user,
             redirect_uri=orm_client.redirect_uri,
             session_id=request.session_id,
         )
         self.db.add(orm_code)
+        orm_code.client = orm_client
+        orm_code.user = request.user.orm_user
         self.db.commit()
 
     def get_authorization_code_scopes(self, client_id, code, redirect_uri, request):
@@ -339,7 +345,7 @@ class JupyterHubRequestValidator(RequestValidator):
         app_log.debug("Saving bearer token %s", log_token)
 
         if request.user is None:
-            raise ValueError("No user for access token: %s" % request.user)
+            raise ValueError(f"No user for access token: {request.user}")
         client = (
             self.db.query(orm.OAuthClient)
             .filter_by(identifier=request.client.client_id)
@@ -419,6 +425,7 @@ class JupyterHubRequestValidator(RequestValidator):
         if orm_client is None:
             return False
         if not orm_client.secret:
+            app_log.warning("OAuth client %s present without secret", client_id)
             return False
         request.client = orm_client
         return True
@@ -551,7 +558,6 @@ class JupyterHubRequestValidator(RequestValidator):
             - Resource Owner Password Credentials Grant
             - Client Credentials Grant
         """
-
         orm_client = (
             self.db.query(orm.OAuthClient).filter_by(identifier=client_id).one_or_none()
         )
@@ -591,19 +597,23 @@ class JupyterHubRequestValidator(RequestValidator):
 
         client_allowed_scopes = set(orm_client.allowed_scopes)
 
+        # scope resolution only works if we have a user defined
+        user = request.user or getattr(self, "_current_user")
+
         # always grant reading the token-owner's name
         # and accessing the service itself
         required_scopes = {*identify_scopes(), *access_scopes(orm_client)}
         requested_scopes.update(required_scopes)
         client_allowed_scopes.update(required_scopes)
 
-        # TODO: handle expanded_scopes intersection here?
-        # e.g. client allowed to request admin:users,
-        # but requests admin:users!name=x will not be allowed
-        # This can probably be dealt with in config by listing expected requests
-        # as explcitly allowed
+        allowed_scopes, disallowed_scopes = _resolve_requested_scopes(
+            requested_scopes,
+            client_allowed_scopes,
+            user=user.orm_user,
+            client=orm_client,
+            db=self.db,
+        )
 
-        disallowed_scopes = requested_scopes.difference(client_allowed_scopes)
         if disallowed_scopes:
             app_log.error(
                 f"Scope(s) not allowed for {client_id}: {', '.join(disallowed_scopes)}"
@@ -660,6 +670,18 @@ class JupyterHubOAuthServer(WebApplicationServer):
         self.db.commit()
         return orm_client
 
+    def remove_client(self, client_id):
+        """Remove a client by its id if it is existed."""
+        orm_client = (
+            self.db.query(orm.OAuthClient).filter_by(identifier=client_id).one_or_none()
+        )
+        if orm_client is not None:
+            self.db.delete(orm_client)
+            self.db.commit()
+            app_log.info("Removed client %s", client_id)
+        else:
+            app_log.warning("No such client %s", client_id)
+
     def fetch_by_client_id(self, client_id):
         """Find a client by its id"""
         client = self.db.query(orm.OAuthClient).filter_by(identifier=client_id).first()
