@@ -1,4 +1,5 @@
 """Some general objects for use in JupyterHub"""
+
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
 import socket
@@ -162,11 +163,9 @@ class Server(HasTraits):
         return f"{self.host}{self.base_url}"
 
     def __repr__(self):
-        return "{name}(url={url}, bind_url={bind})".format(
-            name=self.__class__.__name__, url=self.url, bind=self.bind_url
-        )
+        return f"{self.__class__.__name__}(url={self.url}, bind_url={self.bind_url})"
 
-    def wait_up(self, timeout=10, http=False, ssl_context=None):
+    def wait_up(self, timeout=10, http=False, ssl_context=None, extra_path=""):
         """Wait for this server to come up"""
         if http:
             ssl_context = ssl_context or make_ssl_context(
@@ -174,7 +173,9 @@ class Server(HasTraits):
             )
 
             return wait_for_http_server(
-                self.url, timeout=timeout, ssl_context=ssl_context
+                url_path_join(self.url, extra_path),
+                timeout=timeout,
+                ssl_context=ssl_context,
             )
         else:
             return wait_for_server(
