@@ -38,10 +38,11 @@ A hub-managed service with no URL::
     }
 
 """
+
 import asyncio
 import copy
 import os
-import pipes
+import shlex
 import shutil
 from subprocess import Popen
 
@@ -52,8 +53,10 @@ from traitlets import (
     HasTraits,
     Instance,
     List,
+    Set,
     Unicode,
     default,
+    observe,
     validate,
 )
 from traitlets.config import LoggingConfigurable
@@ -130,7 +133,7 @@ class _ServiceSpawner(LocalProcessSpawner):
             env['SYSTEMROOT'] = os.environ['SYSTEMROOT']
         cmd = self.cmd
 
-        self.log.info("Spawning %s", ' '.join(pipes.quote(s) for s in cmd))
+        self.log.info("Spawning %s", ' '.join(shlex.quote(s) for s in cmd))
         try:
             self.proc = Popen(
                 self.cmd,
@@ -164,6 +167,12 @@ class Service(LoggingConfigurable):
     - url: str (None)
         The URL where the service is/should be.
         If specified, the service will be added to the proxy at /services/:name
+    - oauth_redirect_url: str ('/services/:name/oauth_redirect')
+        The URI for the oauth redirect.
+
+        Not usually needed, but must be set for external services that are not accessed through the proxy,
+        or any service which have a redirect URI different from the default of `/services/:name/oauth_redirect`.
+
     - oauth_no_confirm: bool(False)
         Whether this service should be allowed to complete oauth
         with logged-in users without prompting for confirmation.
@@ -174,15 +183,18 @@ class Service(LoggingConfigurable):
         Command for JupyterHub to spawn the service.
         Only use this if the service should be a subprocess.
         If command is not specified, it is assumed to be managed
-        by a
+        by an external process.
     - environment: dict
         Additional environment variables for the service.
     - user: str
         The name of a system user to become.
         If unspecified, run as the same user as the Hub.
+
     """
 
-    # inputs:
+    # traits tagged with `input=True` are accepted as input from configuration / API
+    # input traits are also persisted to the db UNLESS they are also tagged with `in_db=False`
+
     name = Unicode(
         help="""The name of the service.
 
@@ -205,7 +217,7 @@ class Service(LoggingConfigurable):
 
         DEPRECATED in 3.0: use oauth_client_allowed_scopes
       """
-    ).tag(input=True)
+    ).tag(input=True, in_db=False)
 
     oauth_client_allowed_scopes = List(
         help="""OAuth allowed scopes.
@@ -225,7 +237,7 @@ class Service(LoggingConfigurable):
 
         If unspecified, an API token will be generated for managed services.
         """
-    ).tag(input=True)
+    ).tag(input=True, in_db=False)
 
     info = Dict(
         help="""Provide a place to include miscellaneous information about the service,
@@ -303,6 +315,7 @@ class Service(LoggingConfigurable):
     cookie_options = Dict()
 
     oauth_provider = Any()
+    _oauth_specified = Set(help="List of oauth config fields specified via config.")
 
     oauth_client_id = Unicode(
         help="""OAuth client ID for this service.
@@ -310,11 +323,11 @@ class Service(LoggingConfigurable):
         You shouldn't generally need to change this.
         Default: `service-<name>`
         """
-    ).tag(input=True)
+    ).tag(input=True, in_db=False)
 
     @default('oauth_client_id')
     def _default_client_id(self):
-        return 'service-%s' % self.name
+        return f'service-{self.name}'
 
     @validate("oauth_client_id")
     def _validate_client_id(self, proposal):
@@ -328,10 +341,12 @@ class Service(LoggingConfigurable):
     oauth_redirect_uri = Unicode(
         help="""OAuth redirect URI for this service.
 
-        You shouldn't generally need to change this.
+        Must be set for external services that are not accessed through the proxy,
+        or any service which have a redirect URI different from the default.
+
         Default: `/services/:name/oauth_callback`
         """
-    ).tag(input=True)
+    ).tag(input=True, in_db=False)
 
     @default('oauth_redirect_uri')
     def _default_redirect_uri(self):
@@ -339,13 +354,23 @@ class Service(LoggingConfigurable):
             return ''
         return self.host + url_path_join(self.prefix, 'oauth_callback')
 
+    @observe("oauth_client_id", "oauth_redirect_uri", "oauth_no_confirm")
+    def _oauth_config_set(self, change):
+        # record that some oauth config is specified
+        if change.new in {"", None}:
+            # this shouldn't happen, but empty values
+            # may sometimes be set to 'un-set' config
+            self._oauth_specified.pop(change.name, None)
+        else:
+            self._oauth_specified.add(change.name)
+
     @property
     def oauth_available(self):
         """Is OAuth available for this client?
 
-        Returns True if a server is defined or oauth_redirect_uri is specified manually
+        Returns True if a server is defined or any oauth config is provided explicitly
         """
-        return bool(self.server is not None or self.oauth_redirect_uri)
+        return bool(self.server is not None or self._oauth_specified)
 
     @property
     def oauth_client(self):
@@ -362,6 +387,14 @@ class Service(LoggingConfigurable):
     def prefix(self):
         return url_path_join(self.base_url, 'services', self.name + '/')
 
+    @property
+    def href(self):
+        """Convenient 'href' to use for links to this service"""
+        if self.domain:
+            return f"//{self.domain}{self.prefix}"
+        else:
+            return self.prefix
+
     @property
     def proxy_spec(self):
         if not self.server:
@@ -371,6 +404,11 @@ class Service(LoggingConfigurable):
         else:
             return self.server.base_url
 
+    @property
+    def from_config(self):
+        """Is the service defined from config file?"""
+        return self.orm.from_config
+
     def __repr__(self):
         return "<{cls}(name={name}{managed})>".format(
             cls=self.__class__.__name__,
@@ -381,7 +419,7 @@ class Service(LoggingConfigurable):
     async def start(self):
         """Start a managed service"""
         if not self.managed:
-            raise RuntimeError("Cannot start unmanaged service %s" % self)
+            raise RuntimeError(f"Cannot start unmanaged service {self}")
         self.log.info("Starting service %r: %r", self.name, self.command)
         env = {}
         env.update(self.environment)
@@ -435,7 +473,7 @@ class Service(LoggingConfigurable):
         """Stop a managed service"""
         self.log.debug("Stopping service %s", self.name)
         if not self.managed:
-            raise RuntimeError("Cannot stop unmanaged service %s" % self)
+            raise RuntimeError(f"Cannot stop unmanaged service {self}")
         if self.spawner:
             if self.orm.server:
                 self.db.delete(self.orm.server)
