@@ -1,4 +1,4 @@
-from .app import main
+from . import main
 
 if __name__ == '__main__':
     main()
