@@ -1,22 +1,24 @@
 """Test scopes for API handlers"""
+
+import types
 from operator import itemgetter
 from unittest import mock
 
 import pytest
 from pytest import mark
 from tornado import web
-from tornado.httputil import HTTPServerRequest
 
 from .. import orm, roles, scopes
 from .._memoize import FrozenDict
-from ..handlers import BaseHandler
+from ..apihandlers import APIHandler
 from ..scopes import (
     Scope,
-    _check_scope_access,
     _expand_self_scope,
     _intersect_expanded_scopes,
+    _resolve_requested_scopes,
     expand_scopes,
     get_scopes_for,
+    has_scope,
     identify_scopes,
     needs_scope,
     parse_scopes,
@@ -25,7 +27,8 @@ from .utils import add_user, api_request, auth_header
 
 
 def get_handler_with_scopes(scopes):
-    handler = mock.Mock(spec=BaseHandler)
+    handler = mock.Mock(spec=APIHandler)
+    handler.has_scope = types.MethodType(APIHandler.has_scope, handler)
     handler.parsed_scopes = parse_scopes(scopes)
     return handler
 
@@ -54,47 +57,39 @@ def test_scope_precendence():
 
 def test_scope_check_present():
     handler = get_handler_with_scopes(['read:users'])
-    assert _check_scope_access(handler, 'read:users')
-    assert _check_scope_access(handler, 'read:users', user='maeby')
+    assert handler.has_scope('read:users')
+    assert handler.has_scope('read:users!user=maeby')
 
 
 def test_scope_check_not_present():
     handler = get_handler_with_scopes(['read:users!user=maeby'])
-    assert _check_scope_access(handler, 'read:users')
-    with pytest.raises(web.HTTPError):
-        _check_scope_access(handler, 'read:users', user='gob')
-    with pytest.raises(web.HTTPError):
-        _check_scope_access(handler, 'read:users', user='gob', server='server')
+    assert not handler.has_scope('read:users')
+    assert not handler.has_scope('read:users!user=gob')
+    assert not handler.has_scope('read:users!server=gob/server')
 
 
 def test_scope_filters():
     handler = get_handler_with_scopes(
         ['read:users', 'read:users!group=bluths', 'read:users!user=maeby']
     )
-    assert _check_scope_access(handler, 'read:users', group='bluth')
-    assert _check_scope_access(handler, 'read:users', user='maeby')
-
-
-def test_scope_multiple_filters():
-    handler = get_handler_with_scopes(['read:users!user=george_michael'])
-    assert _check_scope_access(
-        handler, 'read:users', user='george_michael', group='bluths'
-    )
+    assert handler.has_scope('read:users!group=bluth')
+    assert handler.has_scope('read:users!user=maeby')
 
 
 def test_scope_parse_server_name():
     handler = get_handler_with_scopes(
         ['servers!server=maeby/server1', 'read:users!user=maeby']
     )
-    assert _check_scope_access(handler, 'servers', user='maeby', server='server1')
+    assert handler.has_scope('servers!server=maeby/server1')
 
 
 class MockAPIHandler:
     def __init__(self):
         self.expanded_scopes = {'users'}
         self.parsed_scopes = {}
-        self.request = mock.Mock(spec=HTTPServerRequest)
+        self.request = mock.Mock(spec=APIHandler)
         self.request.path = '/path'
+        self.db = None
 
     def set_scopes(self, *scopes):
         self.expanded_scopes = set(scopes)
@@ -293,8 +288,8 @@ async def test_exceeding_user_permissions(
     api_token = user.new_api_token()
     orm_api_token = orm.APIToken.find(app.db, token=api_token)
     # store scopes user does not have
-    orm_api_token.scopes = orm_api_token.scopes + ['list:users', 'read:users']
-    headers = {'Authorization': 'token %s' % api_token}
+    orm_api_token.scopes = list(orm_api_token.scopes) + ['list:users', 'read:users']
+    headers = {'Authorization': f'token {api_token}'}
     r = await api_request(app, 'users', headers=headers)
     assert r.status_code == 200
     keys = {key for user in r.json() for key in user.keys()}
@@ -312,7 +307,7 @@ async def test_user_service_separation(app, mockservice_url, create_temp_role):
     roles.update_roles(app.db, mockservice_url.orm, roles=['reader_role'])
     user.roles.remove(orm.Role.find(app.db, name='user'))
     api_token = user.new_api_token()
-    headers = {'Authorization': 'token %s' % api_token}
+    headers = {'Authorization': f'token {api_token}'}
     r = await api_request(app, 'users', headers=headers)
     assert r.status_code == 200
     keys = {key for user in r.json() for key in user.keys()}
@@ -382,7 +377,7 @@ async def test_user_filter_with_group(app, create_user_with_scopes):
         group = orm.Group(name=group_name)
         app.db.add(group)
     for user in {user1, user2}:
-        group.users.append(user)
+        group.users.append(user.orm_user)
     app.db.commit()
 
     r = await api_request(app, 'users', headers=auth_header(app.db, user1.name))
@@ -488,7 +483,7 @@ async def test_metascope_inherit_expansion(app, create_user_with_scopes):
     assert user_scope_set == token_scope_set
 
     # Check no roles means no permissions
-    token.scopes.clear()
+    token.scopes = []
     app.db.commit()
     token_scope_set = get_scopes_for(token)
     assert isinstance(token_scope_set, frozenset)
@@ -547,28 +542,35 @@ async def test_server_state_access(
         if not group:
             group = orm.Group(name=group_name)
             app.db.add(group)
-        group.users.append(user)
+        group.users.append(user.orm_user)
         app.db.commit()
         server_names = ['bianca', 'terry']
         for server_name in server_names:
             await api_request(
                 app, 'users', user.name, 'servers', server_name, method='post'
             )
-        service = create_service_with_scopes(*scopes)
+        service = create_service_with_scopes("read:users:name!user=bianca", *scopes)
         api_token = service.new_api_token()
-        headers = {'Authorization': 'token %s' % api_token}
+        headers = {'Authorization': f'token {api_token}'}
+
+        # can I get the user model?
         r = await api_request(app, 'users', user.name, headers=headers)
-        r.raise_for_status()
-        user_model = r.json()
-        if num_servers:
-            assert 'servers' in user_model
-            server_models = user_model['servers']
-            assert len(server_models) == num_servers
-            for server, server_model in server_models.items():
-                assert keys_in.issubset(server_model)
-                assert keys_out.isdisjoint(server_model)
+        can_read_user_model = num_servers > 1 or 'read:users' in scopes
+        if can_read_user_model:
+            r.raise_for_status()
+            user_model = r.json()
+            if num_servers > 1:
+                assert 'servers' in user_model
+                server_models = user_model['servers']
+                assert len(server_models) == num_servers
+                for server, server_model in server_models.items():
+                    assert keys_in.issubset(server_model)
+                    assert keys_out.isdisjoint(server_model)
+            else:
+                assert 'servers' not in user_model
         else:
-            assert 'servers' not in user_model
+            assert r.status_code == 404
+
         r = await api_request(
             app,
             'users',
@@ -747,6 +749,7 @@ async def test_service_model_filtering(
                 'name',
                 'kind',
                 'users',
+                'properties',
             },
         ),
         (
@@ -903,6 +906,22 @@ def test_intersect_expanded_scopes(left, right, expected, should_warn, recwarn):
             ["read:users!user=uy"],
             {"gx": ["ux"], "gy": ["uy"]},
         ),
+        (
+            # make sure the group > user > server hierarchy
+            # is managed
+            ["read:servers!server=ux/server", "read:servers!group=gy"],
+            ["read:servers!server=uy/server", "read:servers!user=ux"],
+            ["read:servers!server=ux/server", "read:servers!server=uy/server"],
+            {"gx": ["ux"], "gy": ["uy"]},
+        ),
+        (
+            # make sure the group > user hierarchy
+            # is managed
+            ["read:servers!user=ux", "read:servers!group=gy"],
+            ["read:servers!user=uy", "read:servers!group=gx"],
+            ["read:servers!user=ux", "read:servers!user=uy"],
+            {"gx": ["ux"], "gy": ["uy"]},
+        ),
     ],
 )
 def test_intersect_groups(request, db, left, right, expected, groups):
@@ -962,11 +981,10 @@ def test_intersect_groups(request, db, left, right, expected, groups):
 async def test_list_users_filter(
     app, group, create_service_with_scopes, scopes, expected
 ):
-
     # create users:
     for i in (1, 2):
         user = add_user(app.db, app, name=f'in-{i}')
-        group.users.append(user)
+        group.users.append(user.orm_user)
         add_user(app.db, app, name=f'out-{i}')
     app.db.commit()
 
@@ -1019,7 +1037,6 @@ async def test_list_users_filter(
 async def test_list_groups_filter(
     request, app, create_service_with_scopes, scopes, expected
 ):
-
     # create groups:
     groups = []
     for i in (1, 2, 3):
@@ -1200,3 +1217,149 @@ def test_expand_scopes(app, user, scopes, expected, mockservice_external):
     expanded = expand_scopes(scopes, owner=user.orm_user, oauth_client=oauth_client)
     assert isinstance(expanded, frozenset)
     assert sorted(expanded) == sorted(expected)
+
+
+@pytest.mark.parametrize(
+    "requested_scopes, have_scopes, expected_allowed, expected_disallowed",
+    [
+        (
+            ["read:users:name!user"],
+            ["read:users:name!user={user}"],
+            ["read:users:name!user"],
+            [],
+        ),
+        (
+            ["read:servers!server"],
+            ["read:servers!user"],
+            ["read:servers!server"],
+            [],
+        ),
+        (
+            ["read:servers!server={server}"],
+            ["read:servers"],
+            ["read:servers!server={server}"],
+            [],
+        ),
+        (
+            ["admin:servers!server"],
+            ["read:servers"],
+            ["read:servers!server={server}"],
+            ["admin:servers!server"],
+        ),
+        (
+            ["admin:servers", "read:users"],
+            ["read:users"],
+            ["read:users"],
+            ["admin:servers"],
+        ),
+    ],
+)
+def test_resolve_requested_scopes(
+    app,
+    user,
+    group,
+    requested_scopes,
+    have_scopes,
+    expected_allowed,
+    expected_disallowed,
+    mockservice_external,
+):
+    if isinstance(requested_scopes, str):
+        requested_scopes = [requested_scopes]
+
+    db = app.db
+    service = mockservice_external
+    spawner_name = "salmon"
+    server_name = f"{user.name}/{spawner_name}"
+    if '!server' in str(requested_scopes + have_scopes):
+        oauth_client = orm.OAuthClient()
+        db.add(oauth_client)
+        spawner = user.spawners[spawner_name]
+        spawner.orm_spawner.oauth_client = oauth_client
+        db.commit()
+        assert oauth_client.spawner is spawner.orm_spawner
+    else:
+        oauth_client = service.oauth_client
+        assert oauth_client is not None
+
+    def format_scopes(scopes):
+        return {
+            s.format(service=service.name, server=server_name, user=user.name)
+            for s in scopes
+        }
+
+    requested_scopes = format_scopes(requested_scopes)
+    have_scopes = format_scopes(have_scopes)
+    expected_allowed = format_scopes(expected_allowed)
+    expected_disallowed = format_scopes(expected_disallowed)
+
+    allowed, disallowed = _resolve_requested_scopes(
+        requested_scopes,
+        have_scopes,
+        user=user.orm_user,
+        client=oauth_client,
+        db=db,
+    )
+    assert allowed == expected_allowed
+    assert disallowed == expected_disallowed
+
+
+@pytest.mark.parametrize(
+    "scope, have_scopes, ok",
+    [
+        # exact matches
+        ("read:users", "read:users", True),
+        ("read:users!user=USER", "read:users!user=USER", True),
+        ("read:servers!server=USER/x", "read:servers!server=USER/x", True),
+        ("read:groups!group=GROUP", "read:groups!group=GROUP", True),
+        # subscopes
+        ("read:users:name", "read:users", True),
+        # subscopes with matching filter
+        ("read:users:name!user=USER", "read:users!user=USER", True),
+        ("read:users!user=USER", "read:users!group=GROUP", True),
+        ("read:users!user=USER", "read:users", True),
+        ("read:servers!server=USER/x", "read:servers", True),
+        ("read:servers!server=USER/x", "servers!server=USER/x", True),
+        ("read:servers!server=USER/x", "servers!user=USER", True),
+        ("read:servers!server=USER/x", "servers!group=GROUP", True),
+        # shouldn't match
+        ("read:users", "read:users!user=USER", False),
+        ("read:users", "read:users!user=USER", False),
+        ("read:users!user=USER", "read:users!user=other", False),
+        ("read:users!user=USER", "read:users!group=other", False),
+        ("read:servers!server=USER/x", "servers!server=other/x", False),
+        ("read:servers!server=USER/x", "servers!user=other", False),
+        ("read:servers!server=USER/x", "servers!group=other", False),
+        ("servers!server=USER/x", "read:servers!server=USER/x", False),
+    ],
+)
+def test_has_scope(app, user, group, scope, have_scopes, ok):
+    db = app.db
+    user.groups.append(group)
+    db.commit()
+
+    def _sub(scope):
+        return scope.replace("GROUP", group.name).replace("USER", user.name)
+
+    scope = _sub(scope)
+    have_scopes = [_sub(s) for s in have_scopes.split(",")]
+    parsed_scopes = parse_scopes(expand_scopes(have_scopes))
+    assert has_scope(scope, parsed_scopes, db=db) == ok
+
+
+@pytest.mark.parametrize(
+    "scope, have_scopes, ok",
+    [
+        ("read:users", "read:users", True),
+        ("read:users", "read:users!user=x", True),
+        ("read:users", "read:groups!user=x", False),
+        ("read:users!user=x", "read:users!group=y", ValueError),
+    ],
+)
+def test_has_scope_post_filter(scope, have_scopes, ok):
+    have_scopes = have_scopes.split(",")
+    if ok not in (True, False):
+        with pytest.raises(ok):
+            has_scope(scope, have_scopes, post_filter=True)
+    else:
+        assert has_scope(scope, have_scopes, post_filter=True) == ok
