@@ -1,13 +1,11 @@
 """Tests for services"""
+
 import os
 import sys
 from binascii import hexlify
+from contextlib import asynccontextmanager
 from subprocess import Popen
 
-from async_generator import asynccontextmanager
-
-from .. import orm
-from ..roles import roles_to_scopes
 from ..utils import (
     exponential_backoff,
     maybe_future,
