@@ -1,5 +1,5 @@
 /*!
- * Font Awesome Free 6.5.2 by @fontawesome - https://fontawesome.com
+ * Font Awesome Free 6.6.0 by @fontawesome - https://fontawesome.com
  * License - https://fontawesome.com/license/free (Icons: CC BY 4.0, Fonts: SIL OFL 1.1, Code: MIT License)
  * Copyright 2024 Fonticons, Inc.
  */
@@ -9,118 +9,32 @@
   (factory());
 }(this, (function () { 'use strict';
 
-  function ownKeys(object, enumerableOnly) {
-    var keys = Object.keys(object);
-
-    if (Object.getOwnPropertySymbols) {
-      var symbols = Object.getOwnPropertySymbols(object);
-      enumerableOnly && (symbols = symbols.filter(function (sym) {
-        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
-      })), keys.push.apply(keys, symbols);
-    }
-
-    return keys;
-  }
-
-  function _objectSpread2(target) {
-    for (var i = 1; i < arguments.length; i++) {
-      var source = null != arguments[i] ? arguments[i] : {};
-      i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
-        _defineProperty(target, key, source[key]);
-      }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
-        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
-      });
-    }
-
-    return target;
-  }
-
-  function _typeof(obj) {
-    "@babel/helpers - typeof";
-
-    return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) {
-      return typeof obj;
-    } : function (obj) {
-      return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
-    }, _typeof(obj);
-  }
-
-  function _defineProperty(obj, key, value) {
-    if (key in obj) {
-      Object.defineProperty(obj, key, {
-        value: value,
-        enumerable: true,
-        configurable: true,
-        writable: true
-      });
-    } else {
-      obj[key] = value;
-    }
-
-    return obj;
-  }
-
-  function _toConsumableArray(arr) {
-    return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
-  }
-
-  function _arrayWithoutHoles(arr) {
-    if (Array.isArray(arr)) return _arrayLikeToArray(arr);
-  }
-
-  function _iterableToArray(iter) {
-    if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
-  }
-
-  function _unsupportedIterableToArray(o, minLen) {
-    if (!o) return;
-    if (typeof o === "string") return _arrayLikeToArray(o, minLen);
-    var n = Object.prototype.toString.call(o).slice(8, -1);
-    if (n === "Object" && o.constructor) n = o.constructor.name;
-    if (n === "Map" || n === "Set") return Array.from(o);
-    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
-  }
-
-  function _arrayLikeToArray(arr, len) {
-    if (len == null || len > arr.length) len = arr.length;
-
-    for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];
-
-    return arr2;
-  }
-
-  function _nonIterableSpread() {
-    throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
-  }
-
-  var _WINDOW = {};
-  var _DOCUMENT = {};
+  let _WINDOW = {};
+  let _DOCUMENT = {};
 
   try {
     if (typeof window !== 'undefined') _WINDOW = window;
     if (typeof document !== 'undefined') _DOCUMENT = document;
   } catch (e) {}
 
-  var _ref = _WINDOW.navigator || {},
-      _ref$userAgent = _ref.userAgent,
-      userAgent = _ref$userAgent === void 0 ? '' : _ref$userAgent;
-  var WINDOW = _WINDOW;
-  var DOCUMENT = _DOCUMENT;
-  var IS_BROWSER = !!WINDOW.document;
-  var IS_DOM = !!DOCUMENT.documentElement && !!DOCUMENT.head && typeof DOCUMENT.addEventListener === 'function' && typeof DOCUMENT.createElement === 'function';
-  var IS_IE = ~userAgent.indexOf('MSIE') || ~userAgent.indexOf('Trident/');
+  const {
+    userAgent = ''
+  } = _WINDOW.navigator || {};
+  const WINDOW = _WINDOW;
+  const DOCUMENT = _DOCUMENT;
+  const IS_BROWSER = !!WINDOW.document;
+  const IS_DOM = !!DOCUMENT.documentElement && !!DOCUMENT.head && typeof DOCUMENT.addEventListener === 'function' && typeof DOCUMENT.createElement === 'function';
+  const IS_IE = ~userAgent.indexOf('MSIE') || ~userAgent.indexOf('Trident/');
 
-  var functions = [];
+  const functions = [];
 
-  var listener = function listener() {
+  const listener = function () {
     DOCUMENT.removeEventListener('DOMContentLoaded', listener);
     loaded = 1;
-    functions.map(function (fn) {
-      return fn();
-    });
+    functions.map(fn => fn());
   };
 
-  var loaded = false;
+  let loaded = false;
 
   if (IS_DOM) {
     loaded = (DOCUMENT.documentElement.doScroll ? /^loaded|^c/ : /^loaded|^i|^c/).test(DOCUMENT.readyState);
@@ -133,25 +47,27 @@
   }
 
   function report (_ref) {
-    var nodesTested = _ref.nodesTested,
-        nodesFound = _ref.nodesFound;
-    var timedOutTests = {};
+    let {
+      nodesTested,
+      nodesFound
+    } = _ref;
+    const timedOutTests = {};
 
-    for (var key in nodesFound) {
+    for (let key in nodesFound) {
       if (!(nodesTested.conflict[key] || nodesTested.noConflict[key])) {
         timedOutTests[key] = nodesFound[key];
       }
     }
 
-    var conflictsCount = Object.keys(nodesTested.conflict).length;
+    const conflictsCount = Object.keys(nodesTested.conflict).length;
 
     if (conflictsCount > 0) {
       console.info("%cConflict".concat(conflictsCount > 1 ? 's' : '', " found:"), 'color: darkred; font-size: large');
-      var data = {};
+      const data = {};
 
-      for (var _key in nodesTested.conflict) {
-        var item = nodesTested.conflict[_key];
-        data[_key] = {
+      for (let key in nodesTested.conflict) {
+        const item = nodesTested.conflict[key];
+        data[key] = {
           'tagName': item.tagName,
           'src/href': item.src || item.href || 'n/a',
           'innerText excerpt': item.innerText && item.innerText !== '' ? item.innerText.slice(0, 200) + '...' : '(empty)'
@@ -161,40 +77,40 @@
       console.table(data);
     }
 
-    var noConflictsCount = Object.keys(nodesTested.noConflict).length;
+    const noConflictsCount = Object.keys(nodesTested.noConflict).length;
 
     if (noConflictsCount > 0) {
       console.info("%cNo conflict".concat(noConflictsCount > 1 ? 's' : '', " found with ").concat(noConflictsCount === 1 ? 'this' : 'these', ":"), 'color: green; font-size: large');
-      var _data = {};
+      const data = {};
 
-      for (var _key2 in nodesTested.noConflict) {
-        var _item = nodesTested.noConflict[_key2];
-        _data[_key2] = {
-          'tagName': _item.tagName,
-          'src/href': _item.src || _item.href || 'n/a',
-          'innerText excerpt': _item.innerText && _item.innerText !== '' ? _item.innerText.slice(0, 200) + '...' : '(empty)'
+      for (let key in nodesTested.noConflict) {
+        const item = nodesTested.noConflict[key];
+        data[key] = {
+          'tagName': item.tagName,
+          'src/href': item.src || item.href || 'n/a',
+          'innerText excerpt': item.innerText && item.innerText !== '' ? item.innerText.slice(0, 200) + '...' : '(empty)'
         };
       }
 
-      console.table(_data);
+      console.table(data);
     }
 
-    var timeOutCount = Object.keys(timedOutTests).length;
+    const timeOutCount = Object.keys(timedOutTests).length;
 
     if (timeOutCount > 0) {
       console.info("%cLeftovers--we timed out before collecting test results for ".concat(timeOutCount === 1 ? 'this' : 'these', ":"), 'color: blue; font-size: large');
-      var _data2 = {};
+      const data = {};
 
-      for (var _key3 in timedOutTests) {
-        var _item2 = timedOutTests[_key3];
-        _data2[_key3] = {
-          'tagName': _item2.tagName,
-          'src/href': _item2.src || _item2.href || 'n/a',
-          'innerText excerpt': _item2.innerText && _item2.innerText !== '' ? _item2.innerText.slice(0, 200) + '...' : '(empty)'
+      for (let key in timedOutTests) {
+        const item = timedOutTests[key];
+        data[key] = {
+          'tagName': item.tagName,
+          'src/href': item.src || item.href || 'n/a',
+          'innerText excerpt': item.innerText && item.innerText !== '' ? item.innerText.slice(0, 200) + '...' : '(empty)'
         };
       }
 
-      console.table(_data2);
+      console.table(data);
     }
   }
 
@@ -617,7 +533,7 @@
   });
 
   function md5ForNode(node) {
-    if (null === node || 'object' !== _typeof(node)) return undefined;
+    if (null === node || 'object' !== typeof node) return undefined;
 
     if (node.src) {
       return md5(node.src);
@@ -631,35 +547,31 @@
     }
   }
 
-  var diagScriptId = 'fa-kits-diag';
-  var nodeUnderTestId = 'fa-kits-node-under-test';
-  var md5Attr = 'data-md5';
-  var detectionIgnoreAttr = 'data-fa-detection-ignore';
-  var timeoutAttr = 'data-fa-detection-timeout';
-  var resultsCollectionMaxWaitAttr = 'data-fa-detection-results-collection-max-wait';
+  const diagScriptId = 'fa-kits-diag';
+  const nodeUnderTestId = 'fa-kits-node-under-test';
+  const md5Attr = 'data-md5';
+  const detectionIgnoreAttr = 'data-fa-detection-ignore';
+  const timeoutAttr = 'data-fa-detection-timeout';
+  const resultsCollectionMaxWaitAttr = 'data-fa-detection-results-collection-max-wait';
 
-  var silenceErrors = function silenceErrors(e) {
+  const silenceErrors = e => {
     e.preventDefault();
     e.stopPropagation();
   };
 
   function pollUntil(_ref) {
-    var _ref$fn = _ref.fn,
-        fn = _ref$fn === void 0 ? function () {
-      return true;
-    } : _ref$fn,
-        _ref$initialDuration = _ref.initialDuration,
-        initialDuration = _ref$initialDuration === void 0 ? 1 : _ref$initialDuration,
-        _ref$maxDuration = _ref.maxDuration,
-        maxDuration = _ref$maxDuration === void 0 ? WINDOW.FontAwesomeDetection.timeout : _ref$maxDuration,
-        _ref$showProgress = _ref.showProgress,
-        showProgress = _ref$showProgress === void 0 ? false : _ref$showProgress,
-        progressIndicator = _ref.progressIndicator;
+    let {
+      fn = () => true,
+      initialDuration = 1,
+      maxDuration = WINDOW.FontAwesomeDetection.timeout,
+      showProgress = false,
+      progressIndicator
+    } = _ref;
     return new Promise(function (resolve, reject) {
       // eslint-disable-line compat/compat
       function poll(duration, cumulativeDuration) {
         setTimeout(function () {
-          var result = fn();
+          const result = fn();
 
           if (showProgress) {
             console.info(progressIndicator);
@@ -669,8 +581,8 @@
             // eslint-disable-line no-extra-boolean-cast
             resolve(result);
           } else {
-            var nextDuration = 250;
-            var nextCumulativeDuration = nextDuration + cumulativeDuration;
+            const nextDuration = 250;
+            const nextCumulativeDuration = nextDuration + cumulativeDuration;
 
             if (nextCumulativeDuration <= maxDuration) {
               poll(nextDuration, nextCumulativeDuration);
@@ -686,10 +598,8 @@
   }
 
   function detectWebfontConflicts() {
-    var linkTags = Array.from(DOCUMENT.getElementsByTagName('link')).filter(function (t) {
-      return !t.hasAttribute(detectionIgnoreAttr);
-    });
-    var styleTags = Array.from(DOCUMENT.getElementsByTagName('style')).filter(function (t) {
+    const linkTags = Array.from(DOCUMENT.getElementsByTagName('link')).filter(t => !t.hasAttribute(detectionIgnoreAttr));
+    const styleTags = Array.from(DOCUMENT.getElementsByTagName('style')).filter(t => {
       if (t.hasAttribute(detectionIgnoreAttr)) {
         return false;
       } // If the browser has loaded the FA5 CSS, let's not test that <style> element.
@@ -705,15 +615,15 @@
     });
 
     function runDiag(scriptOrLinkTag, md5) {
-      var diagFrame = DOCUMENT.createElement('iframe'); // Using "visibility: hidden; position: absolute" instead of "display: none;" because
+      const diagFrame = DOCUMENT.createElement('iframe'); // Using "visibility: hidden; position: absolute" instead of "display: none;" because
       // Firefox will not return the expected results for getComputedStyle if our iframe has display: none.
 
       diagFrame.setAttribute('style', 'visibility: hidden; position: absolute; height: 0; width: 0;');
-      var testIconId = 'fa-test-icon-' + md5;
-      var iTag = DOCUMENT.createElement('i');
+      const testIconId = 'fa-test-icon-' + md5;
+      const iTag = DOCUMENT.createElement('i');
       iTag.setAttribute('class', 'fa fa-coffee');
       iTag.setAttribute('id', testIconId);
-      var diagScript = DOCUMENT.createElement('script');
+      const diagScript = DOCUMENT.createElement('script');
       diagScript.setAttribute('id', diagScriptId); // WARNING: this function will be toString()'d and assigned to innerText of the diag script
       // element that we'll be putting into a diagnostic iframe.
       // That means that this code won't compile until after the outer script has run and injected
@@ -722,12 +632,12 @@
       // will probably cause it to choke. Chrome will show an error like this:
       // Uncaught SyntaxError: Unexpected end of input
 
-      var diagScriptFun = function diagScriptFun(nodeUnderTestId, testIconId, md5, parentOrigin) {
+      const diagScriptFun = (nodeUnderTestId, testIconId, md5, parentOrigin) => {
         parent.FontAwesomeDetection.__pollUntil({
-          fn: function fn() {
-            var iEl = document.getElementById(testIconId);
-            var computedStyle = window.getComputedStyle(iEl);
-            var fontFamily = computedStyle.getPropertyValue('font-family');
+          fn: () => {
+            const iEl = document.getElementById(testIconId);
+            const computedStyle = window.getComputedStyle(iEl);
+            const fontFamily = computedStyle.getPropertyValue('font-family');
 
             if (!!fontFamily.match(/FontAwesome/) || !!fontFamily.match(/Font Awesome [56]/)) {
               return true;
@@ -735,18 +645,18 @@
               return false;
             }
           }
-        }).then(function () {
-          var node = document.getElementById(nodeUnderTestId);
+        }).then(() => {
+          const node = document.getElementById(nodeUnderTestId);
           parent.postMessage({
             type: 'fontawesome-conflict',
             technology: 'webfont',
             href: node.href,
             innerText: node.innerText,
             tagName: node.tagName,
-            md5: md5
+            md5
           }, parentOrigin);
         }).catch(function (e) {
-          var node = document.getElementById(nodeUnderTestId);
+          const node = document.getElementById(nodeUnderTestId);
 
           if (e === 'timeout') {
             parent.postMessage({
@@ -755,7 +665,7 @@
               href: node.src,
               innerText: node.innerText,
               tagName: node.tagName,
-              md5: md5
+              md5
             }, parentOrigin);
           } else {
             console.error(e);
@@ -763,7 +673,7 @@
         });
       };
 
-      var parentOrigin = WINDOW.location.origin === 'file://' ? '*' : WINDOW.location.origin;
+      const parentOrigin = WINDOW.location.origin === 'file://' ? '*' : WINDOW.location.origin;
       diagScript.innerText = "(".concat(diagScriptFun.toString(), ")('").concat(nodeUnderTestId, "', '").concat(testIconId || 'foo', "', '").concat(md5, "', '").concat(parentOrigin, "');");
 
       diagFrame.onload = function () {
@@ -773,31 +683,29 @@
         diagFrame.contentDocument.body.appendChild(iTag);
       };
 
-      domready(function () {
-        return DOCUMENT.body.appendChild(diagFrame);
-      });
+      domready(() => DOCUMENT.body.appendChild(diagFrame));
     }
 
-    var cssByMD5 = {};
+    const cssByMD5 = {};
 
-    for (var i = 0; i < linkTags.length; i++) {
-      var linkUnderTest = DOCUMENT.createElement('link');
+    for (let i = 0; i < linkTags.length; i++) {
+      const linkUnderTest = DOCUMENT.createElement('link');
       linkUnderTest.setAttribute('id', nodeUnderTestId);
       linkUnderTest.setAttribute('href', linkTags[i].href);
       linkUnderTest.setAttribute('rel', linkTags[i].rel);
-      var md5ForLink = md5ForNode(linkTags[i]);
+      const md5ForLink = md5ForNode(linkTags[i]);
       linkUnderTest.setAttribute(md5Attr, md5ForLink);
       cssByMD5[md5ForLink] = linkTags[i];
       runDiag(linkUnderTest, md5ForLink);
     }
 
-    for (var _i = 0; _i < styleTags.length; _i++) {
+    for (let i = 0; i < styleTags.length; i++) {
       var styleUnderTest = DOCUMENT.createElement('style');
       styleUnderTest.setAttribute('id', nodeUnderTestId);
-      var md5ForStyle = md5ForNode(styleTags[_i]);
+      const md5ForStyle = md5ForNode(styleTags[i]);
       styleUnderTest.setAttribute(md5Attr, md5ForStyle);
-      styleUnderTest.innerText = styleTags[_i].innerText;
-      cssByMD5[md5ForStyle] = styleTags[_i];
+      styleUnderTest.innerText = styleTags[i].innerText;
+      cssByMD5[md5ForStyle] = styleTags[i];
       runDiag(styleUnderTest, md5ForStyle);
     }
 
@@ -805,17 +713,15 @@
   }
 
   function detectSvgConflicts(currentScript) {
-    var scripts = Array.from(DOCUMENT.scripts).filter(function (t) {
-      return !t.hasAttribute(detectionIgnoreAttr) && t !== currentScript;
-    });
-    var scriptsByMD5 = {};
+    const scripts = Array.from(DOCUMENT.scripts).filter(t => !t.hasAttribute(detectionIgnoreAttr) && t !== currentScript);
+    const scriptsByMD5 = {};
 
-    var _loop = function _loop(scriptIdx) {
-      var diagFrame = DOCUMENT.createElement('iframe');
+    for (let scriptIdx = 0; scriptIdx < scripts.length; scriptIdx++) {
+      const diagFrame = DOCUMENT.createElement('iframe');
       diagFrame.setAttribute('style', 'display:none;');
-      var scriptUnderTest = DOCUMENT.createElement('script');
+      const scriptUnderTest = DOCUMENT.createElement('script');
       scriptUnderTest.setAttribute('id', nodeUnderTestId);
-      var md5ForScript = md5ForNode(scripts[scriptIdx]);
+      const md5ForScript = md5ForNode(scripts[scriptIdx]);
       scriptUnderTest.setAttribute(md5Attr, md5ForScript);
       scriptsByMD5[md5ForScript] = scripts[scriptIdx];
 
@@ -828,24 +734,24 @@
       }
 
       scriptUnderTest.async = true;
-      var diagScript = DOCUMENT.createElement('script');
+      const diagScript = DOCUMENT.createElement('script');
       diagScript.setAttribute('id', diagScriptId);
-      var parentOrigin = WINDOW.location.origin === 'file://' ? '*' : WINDOW.location.origin;
+      const parentOrigin = WINDOW.location.origin === 'file://' ? '*' : WINDOW.location.origin;
 
-      var diagScriptFun = function diagScriptFun(nodeUnderTestId, md5, parentOrigin) {
+      const diagScriptFun = (nodeUnderTestId, md5, parentOrigin) => {
         parent.FontAwesomeDetection.__pollUntil({
-          fn: function fn() {
+          fn: () => {
             return !!window.FontAwesomeConfig || !!window.FontAwesomeKitConfig;
           }
         }).then(function () {
-          var scriptNode = document.getElementById(nodeUnderTestId);
+          const scriptNode = document.getElementById(nodeUnderTestId);
           parent.postMessage({
             type: 'fontawesome-conflict',
             technology: 'js',
             src: scriptNode.src,
             innerText: scriptNode.innerText,
             tagName: scriptNode.tagName,
-            md5: md5
+            md5
           }, parentOrigin);
         }).catch(function (e) {
           var scriptNode = document.getElementById(nodeUnderTestId);
@@ -856,7 +762,7 @@
               src: scriptNode.src,
               innerText: scriptNode.innerText,
               tagName: scriptNode.tagName,
-              md5: md5
+              md5
             }, parentOrigin);
           } else {
             console.error(e);
@@ -872,21 +778,17 @@
         diagFrame.contentDocument.head.appendChild(scriptUnderTest);
       };
 
-      domready(function () {
-        return DOCUMENT.body.appendChild(diagFrame);
-      });
-    };
-
-    for (var scriptIdx = 0; scriptIdx < scripts.length; scriptIdx++) {
-      _loop(scriptIdx);
+      domready(() => DOCUMENT.body.appendChild(diagFrame));
     }
 
     return scriptsByMD5;
   }
 
   function setDoneResults(_ref2) {
-    var nodesTested = _ref2.nodesTested,
-        nodesFound = _ref2.nodesFound;
+    let {
+      nodesTested,
+      nodesFound
+    } = _ref2;
     WINDOW.FontAwesomeDetection = WINDOW.FontAwesomeDetection || {};
     WINDOW.FontAwesomeDetection.nodesTested = nodesTested;
     WINDOW.FontAwesomeDetection.nodesFound = nodesFound;
@@ -894,8 +796,8 @@
   }
 
   function conflictDetection() {
-    var report$$1 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : function () {};
-    var nodesTested = {
+    let report$$1 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : () => {};
+    const nodesTested = {
       conflict: {},
       noConflict: {}
     };
@@ -912,19 +814,19 @@
       }
     };
 
-    var scriptsToTest = detectSvgConflicts(DOCUMENT.currentScript);
-    var cssToTest = detectWebfontConflicts();
-
-    var nodesFound = _objectSpread2(_objectSpread2({}, scriptsToTest), cssToTest);
-
-    var testCount = Object.keys(scriptsToTest).length + Object.keys(cssToTest).length; // The resultsCollectionMaxWait allows for the time between when the tests running under
+    const scriptsToTest = detectSvgConflicts(DOCUMENT.currentScript);
+    const cssToTest = detectWebfontConflicts();
+    const nodesFound = { ...scriptsToTest,
+      ...cssToTest
+    };
+    const testCount = Object.keys(scriptsToTest).length + Object.keys(cssToTest).length; // The resultsCollectionMaxWait allows for the time between when the tests running under
     // child iframes call postMessage with their results, and when the parent window
     // receives and handles those events with window.onmessage.
     // Making it configurable allows us to test the scenario where this timeout is exceeded.
     // Naming it something very different from "timeout" is to help avoid the potential ambiguity between
     // these two timeout-related settings.
 
-    var masterTimeout = WINDOW.FontAwesomeDetection.timeout + WINDOW.FontAwesomeDetection.resultsCollectionMaxWait;
+    const masterTimeout = WINDOW.FontAwesomeDetection.timeout + WINDOW.FontAwesomeDetection.resultsCollectionMaxWait;
     console.group('Font Awesome Detector');
 
     if (testCount === 0) {
@@ -941,41 +843,41 @@
         maxDuration: masterTimeout,
         showProgress: true,
         progressIndicator: 'waiting...',
-        fn: function fn() {
+        fn: () => {
           return Object.keys(nodesTested.conflict).length + Object.keys(nodesTested.noConflict).length >= testCount;
         }
-      }).then(function () {
+      }).then(() => {
         console.info('DONE!');
         setDoneResults({
-          nodesTested: nodesTested,
-          nodesFound: nodesFound
+          nodesTested,
+          nodesFound
         });
         report$$1({
-          nodesTested: nodesTested,
-          nodesFound: nodesFound
+          nodesTested,
+          nodesFound
         });
         console.groupEnd();
-      }).catch(function (e) {
+      }).catch(e => {
         if (e === 'timeout') {
           console.info('TIME OUT! We waited until we got tired. Here\'s what we found:');
           setDoneResults({
-            nodesTested: nodesTested,
-            nodesFound: nodesFound
+            nodesTested,
+            nodesFound
           });
           report$$1({
-            nodesTested: nodesTested,
-            nodesFound: nodesFound
+            nodesTested,
+            nodesFound
           });
         } else {
           console.info('Whoops! We hit an error:', e);
           console.info('Here\'s what we\'d found up until that error:');
           setDoneResults({
-            nodesTested: nodesTested,
-            nodesFound: nodesFound
+            nodesTested,
+            nodesFound
           });
           report$$1({
-            nodesTested: nodesTested,
-            nodesFound: nodesFound
+            nodesTested,
+            nodesFound
           });
         }
 
@@ -984,144 +886,222 @@
     }
   } // Allow clients to access, and in some cases, override some properties
 
-  var initialConfig = WINDOW.FontAwesomeDetection || {}; // These can be overridden
+  const initialConfig = WINDOW.FontAwesomeDetection || {}; // These can be overridden
 
-  var _default = {
-    report: report,
+  const _default = {
+    report,
     timeout: +(DOCUMENT.currentScript.getAttribute(timeoutAttr) || "2000"),
     resultsCollectionMaxWait: +(DOCUMENT.currentScript.getAttribute(resultsCollectionMaxWaitAttr) || "5000")
   };
-
-  var _config = _objectSpread2(_objectSpread2(_objectSpread2({}, _default), initialConfig), {}, {
+  const _config = { ..._default,
+    ...initialConfig,
     // These cannot be overridden
     __pollUntil: pollUntil,
-    md5ForNode: md5ForNode,
+    md5ForNode,
     detectionDone: false,
     nodesTested: null,
     nodesFound: null
-  });
-
+  };
   WINDOW.FontAwesomeDetection = _config;
 
-  var _familyProxy, _familyProxy2, _familyProxy3, _familyProxy4, _familyProxy5;
-  var PRODUCTION = function () {
+  var a = "classic",
+      r = "sharp",
+      o = "sharp-duotone";
+
+  var et$1 = {
+    classic: {
+      900: "fas",
+      400: "far",
+      normal: "far",
+      300: "fal",
+      100: "fat"
+    },
+    sharp: {
+      900: "fass",
+      400: "fasr",
+      300: "fasl",
+      100: "fast"
+    },
+    "sharp-duotone": {
+      900: "fasds"
+    }
+  };
+  var bt = {
+    kit: {
+      fak: "kit",
+      "fa-kit": "kit"
+    },
+    "kit-duotone": {
+      fakd: "kit-duotone",
+      "fa-kit-duotone": "kit-duotone"
+    }
+  };
+  var lo = {
+    classic: {
+      "fa-brands": "fab",
+      "fa-duotone": "fad",
+      "fa-light": "fal",
+      "fa-regular": "far",
+      "fa-solid": "fas",
+      "fa-thin": "fat"
+    },
+    sharp: {
+      "fa-solid": "fass",
+      "fa-regular": "fasr",
+      "fa-light": "fasl",
+      "fa-thin": "fast"
+    },
+    "sharp-duotone": {
+      "fa-solid": "fasds"
+    }
+  },
+      no = {
+    classic: {
+      fab: "fa-brands",
+      fad: "fa-duotone",
+      fal: "fa-light",
+      far: "fa-regular",
+      fas: "fa-solid",
+      fat: "fa-thin"
+    },
+    sharp: {
+      fass: "fa-solid",
+      fasr: "fa-regular",
+      fasl: "fa-light",
+      fast: "fa-thin"
+    },
+    "sharp-duotone": {
+      fasds: "fa-solid"
+    }
+  },
+      fo = {
+    classic: {
+      solid: "fas",
+      regular: "far",
+      light: "fal",
+      thin: "fat",
+      duotone: "fad",
+      brands: "fab"
+    },
+    sharp: {
+      solid: "fass",
+      regular: "fasr",
+      light: "fasl",
+      thin: "fast"
+    },
+    "sharp-duotone": {
+      solid: "fasds"
+    }
+  },
+      ho = {
+    classic: {
+      fa: "solid",
+      fas: "solid",
+      "fa-solid": "solid",
+      far: "regular",
+      "fa-regular": "regular",
+      fal: "light",
+      "fa-light": "light",
+      fat: "thin",
+      "fa-thin": "thin",
+      fad: "duotone",
+      "fa-duotone": "duotone",
+      fab: "brands",
+      "fa-brands": "brands"
+    },
+    sharp: {
+      fa: "solid",
+      fass: "solid",
+      "fa-solid": "solid",
+      fasr: "regular",
+      "fa-regular": "regular",
+      fasl: "light",
+      "fa-light": "light",
+      fast: "thin",
+      "fa-thin": "thin"
+    },
+    "sharp-duotone": {
+      fa: "solid",
+      fasds: "solid",
+      "fa-solid": "solid"
+    }
+  };
+  var Io = {
+    kit: {
+      "fa-kit": "fak"
+    },
+    "kit-duotone": {
+      "fa-kit-duotone": "fakd"
+    }
+  },
+      Fo = {
+    kit: {
+      fak: "fa-kit"
+    },
+    "kit-duotone": {
+      fakd: "fa-kit-duotone"
+    }
+  },
+      So = {
+    kit: {
+      kit: "fak"
+    },
+    "kit-duotone": {
+      "kit-duotone": "fakd"
+    }
+  };
+
+  const PRODUCTION = (() => {
     try {
       return process.env.NODE_ENV === 'production';
-    } catch (e) {
+    } catch (e$$1) {
       return false;
     }
-  }();
-  var FAMILY_CLASSIC = 'classic';
-  var FAMILY_SHARP = 'sharp';
-  var FAMILIES = [FAMILY_CLASSIC, FAMILY_SHARP];
+  })();
 
   function familyProxy(obj) {
     // Defaults to the classic family if family is not available
     return new Proxy(obj, {
-      get: function get(target, prop) {
-        return prop in target ? target[prop] : target[FAMILY_CLASSIC];
+      get(target, prop) {
+        return prop in target ? target[prop] : target[a];
       }
+
     });
   }
-  var PREFIX_TO_STYLE = familyProxy((_familyProxy = {}, _defineProperty(_familyProxy, FAMILY_CLASSIC, {
-    'fa': 'solid',
-    'fas': 'solid',
-    'fa-solid': 'solid',
-    'far': 'regular',
-    'fa-regular': 'regular',
-    'fal': 'light',
-    'fa-light': 'light',
-    'fat': 'thin',
-    'fa-thin': 'thin',
-    'fad': 'duotone',
-    'fa-duotone': 'duotone',
-    'fab': 'brands',
-    'fa-brands': 'brands',
-    'fak': 'kit',
-    'fakd': 'kit',
-    'fa-kit': 'kit',
-    'fa-kit-duotone': 'kit'
-  }), _defineProperty(_familyProxy, FAMILY_SHARP, {
-    'fa': 'solid',
-    'fass': 'solid',
-    'fa-solid': 'solid',
-    'fasr': 'regular',
-    'fa-regular': 'regular',
-    'fasl': 'light',
-    'fa-light': 'light',
-    'fast': 'thin',
-    'fa-thin': 'thin'
-  }), _familyProxy));
-  var STYLE_TO_PREFIX = familyProxy((_familyProxy2 = {}, _defineProperty(_familyProxy2, FAMILY_CLASSIC, {
-    solid: 'fas',
-    regular: 'far',
-    light: 'fal',
-    thin: 'fat',
-    duotone: 'fad',
-    brands: 'fab',
-    kit: 'fak'
-  }), _defineProperty(_familyProxy2, FAMILY_SHARP, {
-    solid: 'fass',
-    regular: 'fasr',
-    light: 'fasl',
-    thin: 'fast'
-  }), _familyProxy2));
-  var PREFIX_TO_LONG_STYLE = familyProxy((_familyProxy3 = {}, _defineProperty(_familyProxy3, FAMILY_CLASSIC, {
-    fab: 'fa-brands',
-    fad: 'fa-duotone',
-    fak: 'fa-kit',
-    fal: 'fa-light',
-    far: 'fa-regular',
-    fas: 'fa-solid',
-    fat: 'fa-thin'
-  }), _defineProperty(_familyProxy3, FAMILY_SHARP, {
-    fass: 'fa-solid',
-    fasr: 'fa-regular',
-    fasl: 'fa-light',
-    fast: 'fa-thin'
-  }), _familyProxy3));
-  var LONG_STYLE_TO_PREFIX = familyProxy((_familyProxy4 = {}, _defineProperty(_familyProxy4, FAMILY_CLASSIC, {
-    'fa-brands': 'fab',
-    'fa-duotone': 'fad',
-    'fa-kit': 'fak',
-    'fa-light': 'fal',
-    'fa-regular': 'far',
-    'fa-solid': 'fas',
-    'fa-thin': 'fat'
-  }), _defineProperty(_familyProxy4, FAMILY_SHARP, {
-    'fa-solid': 'fass',
-    'fa-regular': 'fasr',
-    'fa-light': 'fasl',
-    'fa-thin': 'fast'
-  }), _familyProxy4));
-  var FONT_WEIGHT_TO_PREFIX = familyProxy((_familyProxy5 = {}, _defineProperty(_familyProxy5, FAMILY_CLASSIC, {
-    900: 'fas',
-    400: 'far',
-    normal: 'far',
-    300: 'fal',
-    100: 'fat'
-  }), _defineProperty(_familyProxy5, FAMILY_SHARP, {
-    900: 'fass',
-    400: 'fasr',
-    300: 'fasl',
-    100: 'fast'
-  }), _familyProxy5));
-  var oneToTen = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
-  var oneToTwenty = oneToTen.concat([11, 12, 13, 14, 15, 16, 17, 18, 19, 20]);
-  var DUOTONE_CLASSES = {
-    GROUP: 'duotone-group',
-    SWAP_OPACITY: 'swap-opacity',
-    PRIMARY: 'primary',
-    SECONDARY: 'secondary'
+
+  const _PREFIX_TO_STYLE = { ...ho
   };
-  var prefixes = new Set();
-  Object.keys(STYLE_TO_PREFIX[FAMILY_CLASSIC]).map(prefixes.add.bind(prefixes));
-  Object.keys(STYLE_TO_PREFIX[FAMILY_SHARP]).map(prefixes.add.bind(prefixes));
-  var RESERVED_CLASSES = [].concat(FAMILIES, _toConsumableArray(prefixes), ['2xs', 'xs', 'sm', 'lg', 'xl', '2xl', 'beat', 'border', 'fade', 'beat-fade', 'bounce', 'flip-both', 'flip-horizontal', 'flip-vertical', 'flip', 'fw', 'inverse', 'layers-counter', 'layers-text', 'layers', 'li', 'pull-left', 'pull-right', 'pulse', 'rotate-180', 'rotate-270', 'rotate-90', 'rotate-by', 'shake', 'spin-pulse', 'spin-reverse', 'spin', 'stack-1x', 'stack-2x', 'stack', 'ul', DUOTONE_CLASSES.GROUP, DUOTONE_CLASSES.SWAP_OPACITY, DUOTONE_CLASSES.PRIMARY, DUOTONE_CLASSES.SECONDARY]).concat(oneToTen.map(function (n) {
-    return "".concat(n, "x");
-  })).concat(oneToTwenty.map(function (n) {
-    return "w-".concat(n);
-  }));
+  _PREFIX_TO_STYLE[a] = { ...ho[a],
+    ...bt['kit'],
+    ...bt['kit-duotone']
+  };
+  const PREFIX_TO_STYLE = familyProxy(_PREFIX_TO_STYLE);
+  const _STYLE_TO_PREFIX = { ...fo
+  };
+  _STYLE_TO_PREFIX[a] = { ..._STYLE_TO_PREFIX[a],
+    ...So['kit'],
+    ...So['kit-duotone']
+  };
+  const STYLE_TO_PREFIX = familyProxy(_STYLE_TO_PREFIX);
+  const _PREFIX_TO_LONG_STYLE = { ...no
+  };
+  _PREFIX_TO_LONG_STYLE[a] = { ..._PREFIX_TO_LONG_STYLE[a],
+    ...Fo['kit']
+  };
+  const PREFIX_TO_LONG_STYLE = familyProxy(_PREFIX_TO_LONG_STYLE);
+  const _LONG_STYLE_TO_PREFIX = { ...lo
+  };
+  _LONG_STYLE_TO_PREFIX[a] = { ..._LONG_STYLE_TO_PREFIX[a],
+    ...Io['kit']
+  };
+  const LONG_STYLE_TO_PREFIX = familyProxy(_LONG_STYLE_TO_PREFIX);
+  const _FONT_WEIGHT_TO_PREFIX = { ...et$1
+  };
+  const FONT_WEIGHT_TO_PREFIX = familyProxy(_FONT_WEIGHT_TO_PREFIX);
+  const prefixes = new Set();
+  Object.keys(STYLE_TO_PREFIX[a]).map(prefixes.add.bind(prefixes));
+  Object.keys(STYLE_TO_PREFIX[r]).map(prefixes.add.bind(prefixes));
+  Object.keys(STYLE_TO_PREFIX[o]).map(prefixes.add.bind(prefixes));
 
   function bunker(fn) {
     try {
@@ -1129,7 +1109,7 @@
         args[_key - 1] = arguments[_key];
       }
 
-      fn.apply(void 0, args);
+      fn(...args);
     } catch (e) {
       if (!PRODUCTION) {
         throw e;
@@ -1137,7 +1117,7 @@
     }
   }
 
-  bunker(function () {
+  bunker(() => {
     if (IS_BROWSER && IS_DOM) {
       conflictDetection(window.FontAwesomeDetection.report);
     }
